package com.huimv.admin.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.admin.common.utils.Result;
import com.huimv.admin.common.utils.ResultCode;
import com.huimv.admin.entity.EnergyEnvDevice;
import com.huimv.admin.entity.EnvDevice;
import com.huimv.admin.mapper.EnergyEnvDeviceMapper;
import com.huimv.admin.service.IEnergyEnvDeviceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-02-21
 */
@Service
public class EnergyEnvDeviceServiceImpl extends ServiceImpl<EnergyEnvDeviceMapper, EnergyEnvDevice> implements IEnergyEnvDeviceService {
    @Autowired
    private EnergyEnvDeviceMapper energyEnvDeviceMapper;

    @Override
    public Result list(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageSize = paramsMap.get("pageSize");
        String pageNo = paramsMap.get("pageNo");
        if (pageSize==null||pageSize=="") {
            pageSize = "10";
        }
        if (pageNo==null||pageNo=="") {
            pageNo = "1";
        }
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        Page<EnergyEnvDevice> page = new Page(Integer.parseInt(pageNo),Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS,energyEnvDeviceMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result add(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String DeviceName = paramsMap.get("DeviceName");
        String BuildLocation = paramsMap.get("BuildLocation");
        String DeviceBrand = paramsMap.get("DeviceBrand");
        String remark = paramsMap.get("remark");
        if (remark == null || remark == "") {
            remark = null;
        }

        EnergyEnvDevice envDevice = new EnergyEnvDevice();
        envDevice.setFarmId(Integer.parseInt(farmId));
        envDevice.setDeviceName(DeviceName);
        envDevice.setBuildLocation(BuildLocation);
        envDevice.setDeviceBrand(DeviceBrand);
        envDevice.setRemark(remark);
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("device_name", DeviceName).eq("farm_id",farmId);
        EnergyEnvDevice device = energyEnvDeviceMapper.selectOne(queryWrapper);
        if (ObjectUtil.isEmpty(device)) {
            energyEnvDeviceMapper.insert(envDevice);
        } else {
            return new Result(ResultCode.FAIL, "设备名称已存在");
        }
        return new Result(ResultCode.SUCCESS,"添加成功");
    }

    @Override
    public Result edit(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String id = paramsMap.get("id");
        String DeviceName = paramsMap.get("DeviceName");
        String BuildLocation = paramsMap.get("BuildLocation");
        String DeviceBrand = paramsMap.get("DeviceBrand");
        String remark = paramsMap.get("remark");
        if (remark == null || remark == "") {
            remark = null;
        }

        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id);
        EnergyEnvDevice envDevice = energyEnvDeviceMapper.selectOne(queryWrapper);
        envDevice.setFarmId(Integer.parseInt(farmId));
        envDevice.setDeviceName(DeviceName);
        envDevice.setDeviceBrand(DeviceBrand);
        envDevice.setBuildLocation(BuildLocation);
        envDevice.setRemark(remark);

        QueryWrapper<EnergyEnvDevice> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("device_name", DeviceName);
        if (ObjectUtil.isEmpty(energyEnvDeviceMapper.selectOne(queryWrapper1))) {
            energyEnvDeviceMapper.updateById(envDevice);
        } else {
            return new Result(ResultCode.FAIL, "设备名称已存在");
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result delete(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String id = paramsMap.get("id");
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id).eq("farm_id", farmId);
        energyEnvDeviceMapper.delete(queryWrapper);
        return new Result(ResultCode.SUCCESS,"删除成功");
    }


    @Override
    public Result bandingUnitId(HttpServletRequest httpServletRequest, EnergyEnvDevice envDevice) {
        Integer unitId = envDevice.getUnitId();
        int count = this.count(new QueryWrapper<EnergyEnvDevice>().eq("unit_id", unitId));
        if (count>0){
            return new Result(10001,"绑定失败，该栋舍已有设备",false);
        }
        return new Result(10000,"绑定成功",true);
    }
}
